@@ -9,7 +9,8 @@ $ pycat -f head.mat -o brain.mat
 """
 
 # import unittest
-from optparse import OptionParser
+# from optparse import OptionParser
+import argparse
 import sys
 import logging
 logger = logging.getLogger(__name__)
@@ -113,7 +114,6 @@ class Model:
 
             # from PyQt4.QtCore import pyqtRemoveInputHook
             # pyqtRemoveInputHook()
-            # import ipdb; ipdb.set_trace() # BREAKPOINT
 
             # print fv1.shape
             # print fv2.shape
@@ -167,7 +167,6 @@ class Model:
             import scipy.stats
             # from PyQt4.QtCore import pyqtRemoveInputHook
             # pyqtRemoveInputHook()
-            # import ipdb; ipdb.set_trace() # BREAKPOINT
 
             # gaussian_kde works only with floating point types
             self.mdl[cl] = scipy.stats.gaussian_kde(clx.astype(np.float))
@@ -224,7 +223,6 @@ class Model:
         # outsha = sha[:-1]
         # from PyQt4.QtCore import pyqtRemoveInputHook
         # pyqtRemoveInputHook()
-        # import ipdb; ipdb.set_trace() # BREAKPOINT
         if self.modelparams['type'] == 'gmmsame':
 
             px = self.mdl[cl].score(x)
@@ -244,7 +242,6 @@ class Model:
             # px = px.reshape(outsha)
             # from PyQt4.QtCore import pyqtRemoveInputHook
             # pyqtRemoveInputHook()
-            # import ipdb; ipdb.set_trace() # BREAKPOINT
         elif self.modelparams['type'] == 'dpgmm':
             # todo here is a hack
             # dpgmm z nějakého důvodu nefunguje pro naše data
@@ -387,7 +384,6 @@ class ImageGraphCut:
         # ped = py3DSeedEditor.py3DSeedEditor(loseeds)
         # ped.show()
 
-        # import ipdb; ipdb.set_trace() # BREAKPOINT
         return loseeds
 
     def __ms_npenalty_fcn(self, axis, mask, ms_zoom, orig_shape):
@@ -407,12 +403,20 @@ class ImageGraphCut:
         return maskz
 
     def __multiscale_gc(self, pyed):
+        """
+        In first step is performed normal GC. 
+        Second step construct finer grid on edges of segmentation from first
+        step.  
+        """
+        deb = True
         # import py3DSeedEditor as ped
 
+
         from PyQt4.QtCore import pyqtRemoveInputHook
         pyqtRemoveInputHook()
         import scipy
         import scipy.ndimage
+# step 1:  low res GC
         ms_zoom = 8  # 0.125 #self.segparams['scale']
         loseeds = pyed.getSeeds()
         logger.debug("msc " + str(np.unique(loseeds)))
@@ -434,8 +438,10 @@ class ImageGraphCut:
         logger.debug('segmentation')
         logger.debug(str(np.max(self.segmentation)))
         logger.debug(str(np.min(self.segmentation)))
-        seg = 1 - self.segmentation.astype(np.int8)
 
+        seg = 1 - self.segmentation.astype(np.int8)
+        # in seg is now stored low resolution segmentation
+# step 2: discontinuity localization 
         segl = scipy.ndimage.filters.laplace(seg, mode='constant')
         logger.debug(str(np.max(segl)))
         logger.debug(str(np.min(segl)))
@@ -458,11 +464,13 @@ class ImageGraphCut:
 #            import py3DSeedEditor
 #            ped = py3DSeedEditor.py3DSeedEditor(segshape)
 #            ped.show()
+# step 3: indexes of new dual graph
         msinds = self.__multiscale_indexes(seg, img_orig.shape, ms_zoom)
-        print 'msinds', msinds.shape
-        # pd = ped.py3DSeedEditor(msinds)
-        # pd.show()
-        # import ipdb; ipdb.set_trace() # BREAKPOINT
+        logger.debug('msinds ' + str(msinds.shape))
+        if deb:
+            import sed3
+            pd = sed3.sed3(msinds) # ), contour=seg)
+            pd.show()
 
         # intensity values for indexes
         # @TODO compute average values for low resolution
@@ -497,7 +505,13 @@ class ImageGraphCut:
 # tlinks - indexes, data_merge
         ms_values_lin = self.__ordered_values_by_indexes(img_orig, msinds)
         seeds = pyed.getSeeds()
+        # if deb:
+        #     import sed3
+        #     se = sed3.sed3(seeds)
+        #     se.show()
         ms_seeds_lin = self.__ordered_values_by_indexes(seeds, msinds)
+        logger.debug("unique seeds " + str(np.unique(seeds)))
+        logger.debug("unique seeds " + str(np.unique(ms_seeds_lin)))
 
         unariesalt = self.__create_tlinks(ms_values_lin,
                                           self.voxels1, self.voxels2,
@@ -642,7 +656,6 @@ class ImageGraphCut:
             logger.warning('Zoom with different output shape')
         dataout = np.zeros(shape, dtype=dtype)
         shpmin = np.minimum(dataout.shape, shape)
-        # import ipdb; ipdb.set_trace() # BREAKPOINT
 
         dataout[:shpmin[0], :shpmin[1], :shpmin[2]] = datares[
             :shpmin[0], :shpmin[1], :shpmin[2]]
@@ -702,8 +715,9 @@ class ImageGraphCut:
             try:
                 # because of negative problem is as 1 segmented background and
                 # as 0 is segmented foreground
-                from lisa import thresholding_functions
-                newData = thresholding_functions.getPriorityObjects(
+                # import thresholding_functions
+                # newData = thresholding_functions.getPriorityObjects(
+                getPriorityObjects(
                     (1 - res_segm),
                     nObj=-1,
                     seeds=(self.seeds == 1).nonzero(),
@@ -771,6 +785,7 @@ class ImageGraphCut:
 
         # Dobře to fungovalo area_weight = 0.05 a cc = 6 a diference se
         # počítaly z :-1
+
         mdl = Model(modelparams=self.modelparams)
         mdl.trainFromImageAndSeeds(data, seeds, 1)
         mdl.trainFromImageAndSeeds(data, seeds, 2)
@@ -966,6 +981,145 @@ class ImageGraphCut:
 
         return result_labeling
 
+def getPriorityObjects(data, nObj=1, seeds=None, debug=False):
+    """
+
+    Vraceni N nejvetsich objektu.
+        input:
+            data - data, ve kterych chceme zachovat pouze nejvetsi objekty
+            nObj - pocet nejvetsich objektu k vraceni
+            seeds - dvourozmerne pole s umistenim pixelu, ktere chce uzivatel
+                vratit (odpovidaji matici "data")
+
+        returns:
+            data s nejvetsimi objekty
+
+    """
+
+    # Oznaceni dat.
+    # labels - oznacena data.
+    # length - pocet rozdilnych oznaceni.
+    dataLabels, length = scipy.ndimage.label(data)
+
+    logger.info('Olabelovano oblasti: ' + str(length))
+
+    if debug:
+        logger.debug('data labels: ' + str(dataLabels))
+
+    # Uzivatel si nevybral specificke objekty.
+    if (seeds == None):
+
+        logger.info('Vraceni bez seedu')
+        logger.debug('Objekty: ' + str(nObj))
+
+        # Zjisteni nejvetsich objektu.
+        arrayLabelsSum, arrayLabels = areaIndexes(dataLabels, length)
+        # Serazeni labelu podle velikosti oznacenych dat (prvku / ploch).
+        arrayLabelsSum, arrayLabels = selectSort(arrayLabelsSum, arrayLabels)
+
+        returning = None
+        label = 0
+        stop = nObj - 1
+
+        # Budeme postupne prochazet arrayLabels a postupne pridavat jednu
+        # oblast za druhou (od te nejvetsi - mimo nuloveho pozadi) dokud
+        # nebudeme mit dany pocet objektu (nObj).
+        while label <= stop:
+
+            if label >= len(arrayLabels):
+                break
+
+            if arrayLabels[label] != 0:
+                if returning == None:
+                    # "Prvni" iterace
+                    returning = data * (dataLabels == arrayLabels[label])
+                else:
+                    # Jakakoli dalsi iterace
+                    returning = returning + data * \
+                        (dataLabels == arrayLabels[label])
+            else:
+                # Musime prodlouzit hledany interval, protoze jsme narazili na
+                # nulove pozadi.
+                stop = stop + 1
+
+            label = label + 1
+
+            if debug:
+                logger.debug(str(label - 1) + ': ' + str(returning))
+
+        if returning == None:
+            logger.info(
+                'Zadna validni olabelovana data! (DEBUG: returning == None)')
+
+        return returning
+
+    # Uzivatel si vybral specificke objekty (seeds != None).
+    else:
+
+        logger.info('Vraceni se seedy')
+
+        # Zalozeni pole pro ulozeni seedu
+        arrSeed = []
+        # Zjisteni poctu seedu.
+        stop = seeds[0].size
+        tmpSeed = 0
+        dim = numpy.ndim(dataLabels)
+        for index in range(0, stop):
+            # Tady se ukladaji labely na mistech, ve kterych kliknul uzivatel.
+            if dim == 3:
+                # 3D data.
+                tmpSeed = dataLabels[
+                    seeds[0][index], seeds[1][index], seeds[2][index]]
+            elif dim == 2:
+                # 2D data.
+                tmpSeed = dataLabels[seeds[0][index], seeds[1][index]]
+
+            # Tady opet pocitam s tim, ze oznaceni nulou pripada cerne oblasti
+            # (pozadi).
+            if tmpSeed != 0:
+                # Pokud se nejedna o pozadi (cernou oblast), tak se novy seed
+                # ulozi do pole "arrSeed"
+                arrSeed.append(tmpSeed)
+
+        # Pokud existuji vhodne labely, vytvori se nova data k vraceni.
+        # Pokud ne, vrati se "None" typ. { Deprecated: Pokud ne, vrati se cela
+        # nafiltrovana data, ktera do funkce prisla (nedojde k vraceni
+        # specifickych objektu). }
+        if len(arrSeed) > 0:
+
+            # Zbaveni se duplikatu.
+            arrSeed = list(set(arrSeed))
+            if debug:
+                logger.debug('seed list:' + str(arrSeed))
+
+            logger.info(
+                'Ruznych prioritnich objektu k vraceni: ' +
+                str(len(arrSeed))
+            )
+
+            # Vytvoreni vystupu - postupne pricitani dat prislunych specif.
+            # labelu.
+            returning = None
+            for index in range(0, len(arrSeed)):
+
+                if returning == None:
+                    returning = data * (dataLabels == arrSeed[index])
+                else:
+                    returning = returning + data * \
+                        (dataLabels == arrSeed[index])
+
+                if debug:
+                    logger.debug((str(index)) + ':' + str(returning))
+
+            return returning
+
+        else:
+
+            logger.info(
+                'Zadna validni data k vraceni - zadne prioritni objekty ' +
+                'nenalezeny (DEBUG: function getPriorityObjects:' +
+                str(len(arrSeed) == 0))
+            return None
 # class Tests(unittest.TestCase):
 #     def setUp(self):
 #         pass
@@ -1002,6 +1156,7 @@ help = {
     'in_file': 'input *.mat file with "data" field',
     'out_file': 'store the output matrix to the file',
     'debug': 'debug mode',
+    'debug_interactivity': "turn on interactive debug mode",
     'test': 'run unit test',
 }
 # @profile
@@ -1018,36 +1173,45 @@ def main():
     ch.setFormatter(formatter)
     logger.addHandler(ch)
 
-    parser = OptionParser(description='Organ segmentation')
-    parser.add_option('-d', '--debug', action='store_true',
-                      dest='debug', help=help['debug'])
-    parser.add_option('-f', '--filename', action='store',
-                      dest='in_filename', default=None,
-                      help=help['in_file'])
-    parser.add_option('-t', '--tests', action='store_true',
-                      dest='unit_test', help=help['test'])
-    parser.add_option('-o', '--outputfile', action='store',
-                      dest='out_filename', default='output.mat',
-                      help=help['out_file'])
-    (options, args) = parser.parse_args()
+    # parser = OptionParser(description='Organ segmentation')
+
+    parser = argparse.ArgumentParser(
+        description=__doc__
+    )
+    parser.add_argument('-d', '--debug', action='store_true',
+                        help=help['debug'])
+    parser.add_argument('-di', '--debug-interactivity', action='store_true',
+                        help=help['debug_interactivity'])
+    parser.add_argument('-i', '--input-file', action='store',
+                        default=None,
+                        help=help['in_file'])
+    parser.add_argument('-t', '--tests', action='store_true',
+                        help=help['test'])
+    parser.add_argument('-o', '--outputfile', action='store',
+                        dest='out_filename', default='output.mat',
+                        help=help['out_file'])
+    # (options, args) = parser.parse_args()
+    options = parser.parse_args()
 
     debug_images = False
 
     if options.debug:
         logger.setLevel(logging.DEBUG)
-        debug_images = True
         # print DEBUG
         # DEBUG = True
 
+    if options.debug_interactivity:
+        debug_images = True
+
     # if options.tests:
     #     sys.argv[1:]=[]
     #     unittest.main()
 
-    if options.in_filename is None:
+    if options.input_file is None:
         raise IOError('No input data!')
 
     else:
-        dataraw = loadmat(options.in_filename,
+        dataraw = loadmat(options.input_file,
                           variable_names=['data', 'voxelsize_mm'])
     # import pdb; pdb.set_trace() # BREAKPOINT
 
@@ -1062,9 +1226,9 @@ def main():
                         # , modelparams={'type': 'gaussian_kde', 'params': []}
                         # , modelparams={'type':'kernel', 'params':[]}  #noqa not in  old scipy
                         # , modelparams={'type':'gmmsame', 'params':{'cvtype':'full', 'n_components':3}} # noqa 3 components
-                        # , segparams={'type': 'multiscale_gc'}  # multisc gc
+                        , segparams={'type': 'multiscale_gc'}  # multisc gc
                         # , modelparams={'fv_type': 'fv001'}
-                        , modelparams={'type': 'dpgmm', 'params': {'cvtype': 'full', 'n_components': 5, 'alpha': 10}}  # noqa 3 components
+                        # , modelparams={'type': 'dpgmm', 'params': {'cvtype': 'full', 'n_components': 5, 'alpha': 10}}  # noqa 3 components
                         )
     igc.interactivity()
 
